#include "cartes.h"
#include <Imagine/Graphics.h>
using namespace Imagine;

const int trefle=0, carreau=1, coeur=2, pique=3;

bool operator==(Carte c1, Carte c2) {
    return c1.val==c2.val && c1.coul==c2.coul;
}

void dessineTrefle(int x, int y) {
    fillCircle(x-taille/2, y, taille/2, BLACK);
    fillCircle(x+taille/2, y, taille/2, BLACK);
    fillCircle(x, y-taille/2, taille/2, BLACK);
    drawLine(x,y-taille/2, x,y+taille/2, BLACK, 3);
}

void dessineCarreau(int x, int y) {
    int pt[] = {x-taille,y, x,y-taille, x+taille,y, x,y+taille};
    fillPoly(pt, 4, RED);
}

void dessineCoeur(int x, int y) {
    fillCircle(x-taille/3, y, taille/2, RED);
    fillCircle(x+taille/3, y, taille/2, RED);
    int pt[] = {x,y+taille, x-2*taille/3,y+taille/3, x+2*taille/3,y+taille/3};
    fillPoly(pt, 3, RED);
}

void dessinePique(int x, int y) {
    fillCircle(x-taille/3, y, taille/2, BLACK);
    fillCircle(x+taille/3, y, taille/2, BLACK);
    int pt[] = {x,y-taille, x-2*taille/3,y-taille/3, x+2*taille/3,y-taille/3};
    fillPoly(pt, 3, BLACK);
    drawLine(x,y-taille/2, x,y+taille/2, BLACK, 3);
}

void dessineCouleur(int coul, int x, int y) {
    switch(coul) {
    case trefle: dessineTrefle(x,y); break;
    case carreau: dessineCarreau(x,y); break;
    case coeur: dessineCoeur(x,y); break;
    case pique: dessinePique(x,y); break;
    default: assert(false);
    }
}

void dessineCarte(Carte c, int x, int y) {
    const int xStr=x+taille/2, yStr=y+taille*3/2;
    std::string str[] = {"2","3","4","5","6","7","8","9","10","V","D","R", "A"};
    fillRect(x, y, l, h, YELLOW);
    Color col = BLACK;
    if(c.coul==carreau || c.coul==coeur) col = RED;
    drawString(xStr,yStr, str[c.val-2], col);

    if(c.val>10) {
        dessineCouleur(c.coul, x+l/2, y+h/2);
        return;
    }
    if(c.val==2 || c.val==3) {
        dessineCouleur(c.coul, x+l/2, y+h/4);
        dessineCouleur(c.coul, x+l/2, y+3*h/4);
        if(c.val==3) {
            dessineCouleur(c.coul, x+l/2, y+h/2);
        }
        return;
    }
    dessineCouleur(c.coul, x+l/4, y+h/5);
    dessineCouleur(c.coul, x+l*3/4, y+h/5);
    dessineCouleur(c.coul, x+l/4, y+h*4/5);
    dessineCouleur(c.coul, x+l*3/4, y+h*4/5);
    if(c.val==5 || c.val==7 || c.val==9)
        dessineCouleur(c.coul, x+l/2, y+h/2);
    if(c.val==6 || c.val==7) {
        dessineCouleur(c.coul, x+l/4, y+h/2);
        dessineCouleur(c.coul, x+l*3/4, y+h/2);
    }
    if(c.val==8 || c.val==9 || c.val==10) {
        dessineCouleur(c.coul, x+l/4, y+h*2/5);
        dessineCouleur(c.coul, x+l*3/4, y+h*2/5);
        dessineCouleur(c.coul, x+l/4, y+h*3/5);
        dessineCouleur(c.coul, x+l*3/4, y+h*3/5);
    }
    if(c.val==10) {
        dessineCouleur(c.coul, x+l/2, y+h*3/10);
        dessineCouleur(c.coul, x+l/2, y+h*7/10);
    }
}

void dessineMain(Main m, int x, int y) {
    for(int i=0; i<5; i++)
        dessineCarte(m.c[i], x+i*(l+marge), y);
}

Main randMain(Main autres[], int n) {
    Main m;
    for(int i=0; i<5; i++) {
        bool ok=false;
        Carte c;
        while(!ok) {
            ok=true;
            c.val=intRandom(2,14); c.coul=intRandom(trefle,pique);
            for(int j=0; j<i; j++) //Deja tire dans la meme main?
                if(m.c[j]==c)
                    ok=false;
            for(int k=0; k<n; k++) // Deja tire dans une autre main?
                for(int j=0; j<5; j++)
                    if(autres[k].c[j]==c)
                        ok=false;
        }
        m.c[i] = c;
    }
    return m;
}
