#include "combi.h"
#include <algorithm> //std::swap (C++ 98)
#include <utility>   //std::swap (C++ 11)
#include <cassert>

void haute(Combi& c) {
    c.index[0] = 0;
    c.n = 1;
    for(int i=1; i<5; i++)
        if(c.m.c[c.index[0]].val < c.m.c[i].val)
            c.index[0]=i;
}

bool paire(Combi& c) {
    for(int i=0; i<5; i++)
        for(int j=i+1; j<5; j++)
            if(c.m.c[i].val == c.m.c[j].val) {
                c.index[0]=i;
                c.index[1]=j;
                c.n=2;
                return true;
            }
    return false;
}

bool suite(Combi& c) {
    int M = c.m.c[0].val;
    for(int i=1; i<5; i++)
        if(M<c.m.c[i].val)
            M=c.m.c[i].val;
    int k=0;
    for(int i=0; i<5; i++) {
        for(int j=0; j<5; j++)
            if(c.m.c[j].val==M-i) {
                c.index[k++] = j;
                break;
            }
        if(k<=i)
            return false;
    }
    c.n=5;
    return true;
}

bool couleur(Combi& c) {
    int coul = c.m.c[0].coul;
    c.index[0] = 0;
    for(int i=1; i<5; i++) {
        if(c.m.c[i].coul != coul)
            return false;
        c.index[i] = i;
    }
    c.n=5;
    return true;
}

bool quinteflush(Combi& c) {
    return (couleur(c) && suite(c));
}

void histoVal(Main m, int histo[5], int val[5]) {
    for(int i=0; i<5; i++)
        histo[i] = val[i] = 0;
    for(int i=0; i<5; i++) {
        bool trouve=false;
        for(int j=0; j<5; j++)
            if(m.c[i].val==val[j]) {
                ++histo[j];
                trouve=true;
            }
        if(! trouve)
            for(int j=0; j<5; j++)
                if(val[j]==0) {
                    val[j] = m.c[i].val;
                    histo[j] = 1;
                    break;
                }
    }
}

bool carre(Combi& c) {
    int histo[5], val[5];
    histoVal(c.m, histo, val);
    for(int i=1; i<5; i++)
        if(histo[i]==4) { val[0]=val[1]; histo[0]=4; }
    if(histo[0]!=4) return false;
    int k=0;
    for(int i=0; i<5; i++)
        if(c.m.c[i].val == val[0])
            c.index[k++] = i;
    c.n=4;
    return true;
}

bool brelan(Combi& c) {
    int histo[5], val[5];
    histoVal(c.m, histo, val);
    for(int i=1; i<5; i++)
        if(histo[i]==3) { val[0]=val[1]; histo[0]=3; }
    if(histo[0]!=3) return false;
    int k=0;
    for(int i=0; i<5; i++)
        if(c.m.c[i].val == val[0])
            c.index[k++] = i;
    c.n=3;
    return true;
}

bool full(Combi& c) {
    int histo[5], val[5];
    histoVal(c.m, histo, val);
    for(int i=1; i<5; i++)
        if(histo[i]==3) { std::swap(val[0],val[i]);
                          std::swap(histo[0],histo[i]); }
    if(histo[0]!=3) return false;
    for(int i=2; i<5; i++)
        if(histo[i]==2) { std::swap(val[1],val[i]);
                          std::swap(histo[1],histo[i]); }
    if(histo[1]!=2) return false;
    int k1=0,k2=3;
    for(int i=0; i<5; i++)
        if(c.m.c[i].val == val[0])
            c.index[k1++] = i;
        else
            c.index[k2++] = i;
    c.n=5;
    return true;
}

bool doublepaire(Combi& c) {
    int histo[5], val[5];
    histoVal(c.m, histo, val);
    for(int i=1; i<5; i++)
        if(histo[i]==2) { std::swap(val[0],val[i]);
                          std::swap(histo[0],histo[i]);
                          break;}
    if(histo[0]!=2) return false;
    for(int i=2; i<5; i++)
        if(histo[i]==2) { std::swap(val[1],val[i]);
                          std::swap(histo[1],histo[i]); }
    if(histo[1]!=2) return false;
    if(val[0]<val[1]) std::swap(val[0],val[1]);
    int k1=0,k2=2;
    for(int i=0; i<5; i++)
        if(c.m.c[i].val == val[0])
            c.index[k1++] = i;
        else if(c.m.c[i].val == val[1])
            c.index[k2++] = i;
    c.n=4;
    return true;
}

void score(Combi& c) {
    c.score=0;
    if(quinteflush(c)) c.score=8;
    else if(carre(c)) c.score=7;
    else if(full(c)) c.score=6;
    else if(couleur(c)) c.score=5;
    else if(suite(c)) c.score=4;
    else if(brelan(c)) c.score=3;
    else if(doublepaire(c)) c.score=2;
    else if(paire(c)) c.score=1;
    else haute(c);
}

int compareCombi(Combi c0, Combi c1) {
    if(c0.score<c1.score)
        return -1;
    if(c0.score>c1.score)
        return 1;
    // Egalite: 1) compare cartes de la combinaison
    assert(c0.n==c1.n);
    for(int i=0; i<c0.n; i++) {
        if(c0.m.c[c0.index[i]].val<c1.m.c[c1.index[i]].val)
            return -1;
        else if(c0.m.c[c0.index[i]].val>c1.m.c[c1.index[i]].val)
            return 1;
        c0.m.c[c0.index[i]].val = c1.m.c[c1.index[i]].val = 0;
    }
    // Egalite: 2) compare cartes restantes par ordre decroissant
    for(int i=0; i<5; i++) // Tri
        for(int j=i+1; j<5; j++) {
            if(c0.m.c[i].val<c0.m.c[j].val)
                std::swap(c0.m.c[i], c0.m.c[j]);
            if(c1.m.c[i].val<c1.m.c[j].val)
                std::swap(c1.m.c[i], c1.m.c[j]);
        }
    for(int i=0; i<5; i++) // Comparaison
        if(c0.m.c[i].val<c1.m.c[i].val)
            return -1;
        else if(c0.m.c[i].val>c1.m.c[i].val)
            return 1;
    return 0;
}
